# -*- coding: utf-8 -*-

import decimal
import os
import sentry_sdk
from importlib import import_module
from typing import Optional

import yaml
from flask import Flask, url_for, request, json, g
from mgdpr.admin import all_admin_views
from mgdpr.admin.admin_home import AdminHome
from mgdpr.blueprints import all_blueprints
from mgdpr.config import Config
from mgdpr.tools import jsonifier

import mgdpr as app_root
from mgdpr.extensions import db, babel, migrate, celery, mail, admin, login_manager

APP_ROOT_FOLDER = os.path.abspath(os.path.dirname(app_root.__file__))
TEMPLATE_FOLDER = os.path.join(APP_ROOT_FOLDER, 'templates')
STATIC_FOLDER = os.path.join(APP_ROOT_FOLDER, 'static')


def get_config(config_class_string: str, yaml_files: Optional[list] = None) -> Config:
    """Load the Flask config from a class.
    Positional arguments:
    config_class_string -- string representation of a configuration class that will be loaded (e.g.
        'mgdpr.config.Production').
    yaml_files -- List of YAML files to load. This is for testing, leave None in dev/production.
    Returns:
    A class object to be fed into app.config.from_object().
    """
    config_module, config_class = config_class_string.rsplit('.', 1)
    config_obj = getattr(import_module(config_module), config_class)

    # Expand some options.
    db_fmt = 'mgdpr.models.{}'
    celery_fmt = 'mgdpr.tasks.{}'
    if getattr(config_obj, 'CELERY_IMPORTS', False):
        config_obj.CELERY_IMPORTS = [celery_fmt.format(m) for m in config_obj.CELERY_IMPORTS]
    for definition in getattr(config_obj, 'CELERYBEAT_SCHEDULE', dict()).values():
        definition.update(task=celery_fmt.format(definition['task']))
    if getattr(config_obj, 'DB_MODELS_IMPORTS', False):
        config_obj.DB_MODELS_IMPORTS = [db_fmt.format(m) for m in config_obj.DB_MODELS_IMPORTS]

    # Load additional configuration settings.
    yaml_files = yaml_files or [f for f in [
        os.path.join('/', 'etc', 'mgdpr', 'config.yml'),
        os.path.abspath(os.path.join(APP_ROOT_FOLDER, '..', 'config.yml')),
        os.path.join(APP_ROOT_FOLDER, 'config.yml'),
    ] if os.path.exists(f)]
    additional_dict = dict()
    for y in yaml_files:
        with open(y) as f:
            loaded_data = yaml.load(f.read(), Loader=yaml.SafeLoader)
            if isinstance(loaded_data, dict):
                additional_dict.update(loaded_data)
            else:
                raise Exception('Failed to parse configuration {}'.format(y))

    # Merge the rest into the Flask app config.
    for key, value in additional_dict.items():
        setattr(config_obj, key, value)

    return config_obj


def create_app(config_obj: Config, no_sql: bool = False) -> Flask:

    """Create an application."""
    app = Flask(__name__, template_folder=TEMPLATE_FOLDER, static_folder=STATIC_FOLDER)
    config_dict = {k: getattr(config_obj, k) for k in dir(config_obj) if not k.startswith('_')}
    app.config.update(config_dict)

    # Import DB models. Flask-SQLAlchemy doesn't do this automatically like Celery does.
    with app.app_context():
        for model in app.config.get('DB_MODELS_IMPORTS', list()):
            import_module(model)

    for bp in all_blueprints:
        import_module(bp.import_name)
        app.register_blueprint(bp)

    class FlaskJSONProvider(json.provider.DefaultJSONProvider):
        """Custom JSONProvider which adds connexion defaults on top of Flask's"""

        @jsonifier.wrap_default
        def default(self, o):
            return super().default(o)

    app.json = FlaskJSONProvider(app)

    def url_for_other_page(page: int) -> str:
        args = request.view_args.copy()
        args['page'] = page
        return url_for(request.endpoint, **args)

    app.jinja_env.globals['url_for_other_page'] = url_for_other_page  # pylint: disable=no-member

    if not no_sql:
        db.init_app(app)

    migrate.init_app(app, db)

    def get_locale() -> str:
        # if a user is logged in, use the locale from the user settings
        user = getattr(g, 'user', None)
        if user is not None:
            return user.locale
        # otherwise try to guess the language from the user accept
        # header the browser transmits.  We support de/fr/en in this
        # example.  The best match wins.
        if request:
            return request.accept_languages.best_match(['cs', 'en'])
        else:
            return app.config.get('BABEL_DEFAULT_LOCALE', 'cs')

    if hasattr(babel, "localeselector"):
        babel.init_app(app)
        babel.localeselector(get_locale)
    else:
        babel.init_app(app, locale_selector=get_locale)

    sentry_sdk.init(
        dsn=app.config.get('SENTRY_CONFIG', {}).get('dsn')
    )

    babel.init_app(app)
    celery.init_app(app)
    mail.init_app(app)
    login_manager.init_app(app)
    admin.init_app(app, index_view=AdminHome())

    with app.app_context():
        for admin_view in all_admin_views:
            cls = getattr(import_module(admin_view['module']), admin_view['cls'])
            admin.add_view(cls())

    with app.app_context():
        import_module('mgdpr.middleware')

    return app
