import sys
import uuid

from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from werkzeug.security import generate_password_hash, check_password_hash

from mgdpr.extensions import db
from mgdpr.tools.sqlalchemy.uuid import GUID

__author__ = "Adam Schubert"
__date__ = "$26.7.2017 19:33:05$"


class BaseTable(db.Model):
    __abstract__ = True
    updated = db.Column(db.DateTime, default=func.now(), onupdate=func.current_timestamp())
    created = db.Column(db.DateTime, default=func.now())


class User(BaseTable):
    __tablename__ = 'user'
    id = db.Column(db.Integer, primary_key=True)
    first_name = db.Column(db.Unicode(255))
    last_name = db.Column(db.Unicode(255))
    email = db.Column(db.Unicode(255), unique=True, nullable=False)
    password = db.Column(db.String(255))

    def set_password(self, password):
        self.password = generate_password_hash(password)

    def check_password(self, password):
        return check_password_hash(self.password, password)

    def is_active(self):
        return True

    def is_authenticated(self):
        return True

    def is_anonymous(self):
        return False

    def get_id(self):
        try:
            if isinstance(self.id, str):
                return self.id
            elif isinstance(self.id, int):
                return self.id
            else:
                return self.id
        except AttributeError:
            raise NotImplementedError('No `id` attribute - override `get_id`')

    def __eq__(self, other):
        """
        Checks the equality of two `UserMixin` objects using `get_id`.
        """
        if isinstance(other, User):
            return self.get_id() == other.get_id()
        return NotImplemented

    def __ne__(self, other):
        """
        Checks the inequality of two `UserMixin` objects using `get_id`.
        """
        equal = self.__eq__(other)
        if equal is NotImplemented:
            return NotImplemented
        return not equal

    def __str__(self):
        return self.email

    if sys.version_info[0] != 2:  # pragma: no cover
        # Python 3 implicitly set __hash__ to None if we override __eq__
        # We set it back to its default implementation
        __hash__ = object.__hash__


class Locale(BaseTable):
    __tablename__ = 'locale'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.Unicode(255), unique=True, nullable=False)
    code = db.Column(db.Unicode(255), unique=True, nullable=False)
    email_translations = relationship(
        "EmailTranslation",
        order_by="EmailTranslation.id",
        backref="locale",
        lazy='dynamic'
    )

    landing_page_translations = relationship(
        "LandingPageTranslation",
        order_by="LandingPageTranslation.id",
        backref="locale",
        lazy='dynamic'
    )

    request_translations = relationship(
        "Request",
        order_by="Request.id",
        backref="locale",
        lazy='dynamic'
    )

    def __str__(self):
        return self.name


class Email(BaseTable):
    __tablename__ = 'email'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.Unicode(255), unique=True, nullable=False)
    from_email = db.Column(db.Unicode(255), nullable=False)
    access_token = db.Column(GUID(), unique=True, nullable=False, default=uuid.uuid4)
    requests = relationship("Request", order_by="Request.id", backref="email", lazy='dynamic')
    email_translations = relationship("EmailTranslation", order_by="EmailTranslation.id", backref="email", lazy='dynamic')

    def __str__(self):
        return self.name


class EmailTranslation(BaseTable):
    __tablename__ = 'email_translation'
    id = db.Column(db.Integer, primary_key=True)
    email_id = db.Column(db.Integer, db.ForeignKey('email.id'), index=True, nullable=False)
    locale_id = db.Column(db.Integer, db.ForeignKey('locale.id'), index=True, nullable=False)
    subject = db.Column(db.Unicode(255), nullable=False)
    body = db.Column(db.UnicodeText, nullable=False)


class Request(BaseTable):
    __tablename__ = 'request'
    id = db.Column(db.Integer, primary_key=True)
    email_id = db.Column(db.Integer, db.ForeignKey('email.id'), index=True, nullable=False)
    locale_id = db.Column(db.Integer, db.ForeignKey('locale.id'), index=True, nullable=False)
    landing_page_id = db.Column(db.Integer, db.ForeignKey('landing_page.id'), index=True, nullable=True)
    payload = db.Column(db.UnicodeText)
    hook_url = db.Column(db.Unicode(255))
    hook_status_code = db.Column(db.Integer)
    hook_response = db.Column(db.UnicodeText)
    uuid = db.Column(GUID(), default=uuid.uuid4)
    target_email = db.Column(db.Unicode(255))
    redirect_url = db.Column(db.UnicodeText, nullable=True)

    def __str__(self):
        return self.uuid


class LandingPage(BaseTable):
    __tablename__ = 'landing_page'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(255), unique=True, nullable=False)
    is_success = db.Column(db.Boolean, nullable=True)
    requests = relationship("Request", order_by="Request.id", backref="landing_page", lazy='dynamic')
    landing_page_translations = relationship(
        "LandingPageTranslation",
        order_by="LandingPageTranslation.id",
        backref="landing_page",
        lazy='dynamic'
    )

    def __str__(self):
        return self.name


class LandingPageTranslation(BaseTable):
    __tablename__ = 'landing_page_translation'
    id = db.Column(db.Integer, primary_key=True)
    landing_page_id = db.Column(db.Integer, db.ForeignKey('landing_page.id'), index=True, nullable=False)
    locale_id = db.Column(db.Integer, db.ForeignKey('locale.id'), index=True, nullable=False)
    text = db.Column(db.UnicodeText)
    title = db.Column(db.Unicode(255))
    meta_description = db.Column(db.Unicode(255))
