from flask_celery import single_instance
import requests
import json
import re
import os
import datetime
from sqlalchemy import or_
from flask import render_template_string
from logging import getLogger
from mgdpr.models.mgdpr import Request, Email, EmailTranslation, Locale, LandingPage
from mgdpr.tools.message import Message
from mgdpr.extensions import celery, db, mail


LOG = getLogger(__name__)


@celery.task(bind=True)
@single_instance(include_args=True)
def send_email(request_id: int) -> None:
    found_request = Request.query.filter_by(id=request_id).first()
    if not found_request:
        raise Exception('Request {} not found'.format(request_id))

    email, email_translation = db.session.query(Email, EmailTranslation) \
        .join(EmailTranslation) \
        .join(Locale) \
        .filter(Email.id == found_request.email.id, Locale.id == found_request.locale.id).first()

    # Preprocces template
    template = re.sub(
        r"{pagelink=(\d+)}",
        r"{{{{url_for('landing_page_index.show_landing_page', landing_page_id=\1, uuid='{}', _external=True)}}}}".format(found_request.uuid),
        email_translation.body
    )

    template = re.sub(
        r"{filelink=(\S+)}",
        lambda m: "{{{{url_for('api_upload.get_file', filename='{}', _external=True)}}}}".format(
            os.path.join(m.group(1))
        ),
        template
    )

    msg = Message(
        email_translation.subject,
        [found_request.target_email],
        html=render_template_string(template),
        sender=email.from_email
    )

    msg.embed_images()

    mail.send(msg)


@celery.task(bind=True)
@single_instance(include_args=True)
def take_out_garbage():
    datetime_from = datetime.datetime.utcnow() - datetime.timedelta(days=7)  # Drop requests older than 7 days
    for request in Request.query.join(LandingPage, isouter=True).filter(
                    Request.created < datetime_from,
            or_(LandingPage.is_success == False, Request.landing_page == None)
    ).all():
        db.session.delete(request)
    db.session.commit()


@celery.task(bind=True)
@single_instance(include_args=True)
def process_land(request_id: int) -> None:
    found_request = Request.query.filter_by(id=request_id).first()
    if not found_request:
        raise Exception('Request {} not found'.format(request_id))

    if not found_request.landing_page:
        raise Exception('Request {} has not landed yet'.format(request_id))

    if found_request.hook_url:
        payload_data = {
            'uuid': str(found_request.uuid),
            'payload': json.loads(found_request.payload),
            'is_success': found_request.landing_page.is_success,
            'locale': found_request.locale.code,
            'email': found_request.target_email
        }

        headers = {'content-type': 'application/json'}

        response = requests.post(found_request.hook_url, data=json.dumps(payload_data), headers=headers)
        found_request.hook_status_code = response.status_code
        found_request.hook_response = response.text

        db.session.add(found_request)
        db.session.commit()
